package com.nisum.order_consumer.controller;

import com.nisum.order_consumer.dto.OrderDto;
import com.nisum.order_consumer.entity.Order;
import com.nisum.order_consumer.service.OrderConsumerService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@RestController
@RequestMapping("/orderConsumer")
public class OrderConsumerController {

    @Autowired
    private OrderConsumerService orderConsumerService;

    private static final Logger logger = LoggerFactory.getLogger(OrderConsumerController.class);




    @PostMapping("/saveOrder")
    public Mono<ResponseEntity<Order>> saveOrder(@RequestBody Mono<Order> orderInfo) {
        return orderInfo.doOnNext(order -> logger.info("Received OrderDto for saving: {}", order))
                .flatMap(order -> orderConsumerService.saveOrder(order))
                .doOnNext(savedOrder -> logger.info("Successfully saved Order: {}", savedOrder))
                .map(ResponseEntity::ok)
                .defaultIfEmpty(ResponseEntity.badRequest().build());
    }

    @GetMapping("/getAllOrders")
    public Flux<Order> getAllOrders() {
        logger.info("Fetching all orders...");
        return orderConsumerService.getAllOrders()
                .doOnNext(order -> logger.info("Fetched Order: {}", order));
    }

    //  Get Order By orderId
    @GetMapping("/getByOrderId/{orderId}")
    public Mono<ResponseEntity<Order>> getOrderById(@PathVariable String orderId) {
        logger.info("Fetching order with ID: {}", orderId);
        return orderConsumerService.getOrderById(orderId)
                .doOnNext(order -> logger.info("Order found: {}", order))
                .map(ResponseEntity::ok)
                .defaultIfEmpty(ResponseEntity.notFound().build());
    }
    @GetMapping("/count/{customerId}")
    public Mono<Long> getOrderCountByCustomerId(@PathVariable String customerId) {
        logger.info("Fetching order count for Customer ID: {}", customerId);
        return orderConsumerService.getOrderCountByCustomerId(customerId)
                .doOnNext(count -> logger.info("Total orders found for Customer ID {}: {}", customerId, count));
    }


}


