package com.nisum.order_consumer.service;

import com.nisum.order_consumer.dto.OrderDto;
import com.nisum.order_consumer.entity.Order;
import com.nisum.order_consumer.mapper.OrderMapper;
import com.nisum.order_consumer.repository.OrderConsumerRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.time.LocalDateTime;
import java.util.List;

@Service
public class OrderConsumerServiceImpl implements OrderConsumerService{

    @Autowired
    private OrderConsumerRepository orderConsumerRepository;

    @Autowired
    private OrderMapper orderMapper;

    @Autowired
    private WebClient webClient;

    private static final Logger logger = LoggerFactory.getLogger(OrderConsumerServiceImpl.class);

    @Override
    public Mono<Order> saveOrder(Order orderInfo) {
        logger.info("Received request to save Order: {}", orderInfo);

        //Order order = orderMapper.toEntity(orderDto);
        orderInfo.setCreatedDate(LocalDateTime.now());
        orderInfo.setModifiedDate(LocalDateTime.now());

        return orderConsumerRepository.save(orderInfo)
                .doOnNext(savedOrder -> logger.info("Order saved successfully: {}", savedOrder));

    }

    @Override
    public Flux<Order> getAllOrders() {
        logger.info("Fetching all orders...");

        return orderConsumerRepository.findAll()
                .doOnNext(order -> logger.info("Fetched Order: {}", order));

    }

    @Override
    public Mono<Order> getOrderById(String orderId) {
        logger.info("Fetching Order with ID: {}", orderId);
        return orderConsumerRepository.findByOrderId(orderId)
                .doOnNext(order -> logger.info("Order found: {}", order));

    }

    @Override
    public Mono<Long> getOrderCountByCustomerId(String customerId) {
        logger.info("Fetching order count for Customer ID: {}", customerId);
        return orderConsumerRepository.countByCustomerId(customerId)
                .doOnNext(count -> logger.info("Total orders for Customer ID {}: {}", customerId, count));
    }
}
